
-- Module: MehTrinketeer
-- Author: Mehran Shahir
-- Description: Automatically tracks and makes the most efficient use of your trinkets

-- global vars

gVersion = "1.12"

gSetupList = {
   [1] = {
      name        = "Default",
      passiveList = {},
      numPassives = 0,
      cooldownList = {},
      numCooldowns = 0,
      autoload = 1,
      zone = 0,
      zonename = ""
   }
}

gCurrSetup = 1

gPassiveTrinkets = {}
gNumPassiveTrinkets = 0
gCooldownTrinkets = {}
gNumCooldownTrinkets = 0
gTrinketsMinWidth = 400
gTrinketsMinHeight = 174
gAdvancedHeight = 76
gPassivesXstart = 10
gPassivesYstart = -39
gCooldownsXstart = 200
gCooldownsYstart = -39
gPaletteXstart = 8
gPaletteYstart = -20
gButtonsScale = 0.6
gButtonNext = 40
gMaxNumCols = 5
gMaxNumRows = 7
gPassivesOn = true
gCooldownsOn = true
gLastPassiveHighlight = -1
gLastCooldownHighlight = -1
gLock1 = false
gLock2 = false
gAutoTrinketEquip1 = false
gAutoTrinketEquip2 = false
gDialogFunc = nil
gEquipADTrinket = false
gAdvancedOpen = false
gMinimapRadius = 78
gRightDragActive = false
gCursorX = 0
gCursorY = 0
gCurrOptBtX = -gMinimapRadius
gCurrOptBtY = 0
gOpts = { AutoEquipCarrot = true, AskArgentDawn = true }

-- my debugging buddy
cnt = 0

WEARING = -1
TRINKETSLOT1 = -1
TRINKETSLOT2 = -2

-- functions

-- copy by value
function MTTCopy(to, from)
   for k,v in pairs(from) do
      if type(v)=="table" then
         to[k] = {}
         MTTCopy(to[k], v);
      else
         to[k] = v;
      end
   end
end

local function MTFindInArray(array, find)

   for i=1,getn(array) do
      if array[i] == find then
         return i
      end
   end

   return 0
end

local function MTFindInLists(itemName)
   
   tobeequipped = {}

   for i = 0,gNumCooldownTrinkets-1 do
      cdstart, cdduration = GetContainerItemCooldown(gCooldownTrinkets[i].bag, gCooldownTrinkets[i].slot)
      if GetTime() > cdstart+cdduration-30 and getn(tobeequipped) < 2 then
         tinsert(tobeequipped, gCooldownTrinkets[i].name)
      end
      if gCooldownTrinkets[i].name == itemName then
         return true, i, "Cooldown", (MTFindInArray(tobeequipped, itemName) > 0)
      end
   end

   for i = 0,gNumPassiveTrinkets-1 do
      cdstart, cdduration = GetContainerItemCooldown(gPassiveTrinkets[i].bag, gPassiveTrinkets[i].slot)
      if GetTime() > cdstart+cdduration-30 and getn(tobeequipped) < 2 then
         tinsert(tobeequipped, gPassiveTrinkets[i].name)
      end
      if gPassiveTrinkets[i].name == itemName then
         return true, i, "Passive", (MTFindInArray(tobeequipped, itemName) > 0)
      end
   end

   return false, -1, "Unknown", false
end

local function MTRemoveFromList(list, len, i)

   -- force copy by value
   removedTrinket          = {}

   removedTrinket.bag      = list[i].bag
   removedTrinket.slot     = list[i].slot
   removedTrinket.name     = list[i].name
   removedTrinket.texture  = list[i].texture

   -- again force copy by value (lua blows by the way)
   for j = i,len-2 do
      list[j].bag       = list[j+1].bag   
      list[j].slot      = list[j+1].slot   
      list[j].name      = list[j+1].name   
      list[j].texture   = list[j+1].texture
   end

   return removedTrinket
end

equipped1frame = -1.0
equipped2frame = -1.0

local function EquipFirstPassive(slot, stop)

   -- if cursor has an object in it, don't mess with it and return failure
   if CursorHasItem() then
      return false
   end

   if stop then
      lastCheck = stop-1
   else
      lastCheck = gNumPassiveTrinkets-1
   end

   if slot == 13 then
      timesinceequip = (updateframe-equipped1frame) + totalupdateelapsed
      canequip = timesinceequip > 1.0
   else
      timesinceequip = (updateframe-equipped2frame) + totalupdateelapsed
      canequip = timesinceequip > 1.0
   end

   if not canequip then
      return false
   end
   
   for i=0,lastCheck do
      if gPassiveTrinkets[i].bag ~= WEARING then
         PickupContainerItem(gPassiveTrinkets[i].bag, gPassiveTrinkets[i].slot)
         PickupInventoryItem(slot)
    
         gPassiveTrinkets[i].bag = WEARING
         gPassiveTrinkets[i].slot = 12 - slot

         updateframe = 0.0
         
         if slot == 13 then
            equipped1frame = updateframe
         else
            equipped2frame = updateframe
         end            
         return true
      end
   end

   return false
end

local function EquipFirstCooldown(slot, stop)

   -- if cursor has an object in it, don't mess with it and return failure
   if CursorHasItem() then
      return false
   end

   if stop then
      lastCheck = stop-1
   else
      lastCheck = gNumCooldownTrinkets-1
   end

   if slot == 13 then
      timesinceequip = (updateframe-equipped1frame) + totalupdateelapsed
      canequip = timesinceequip > 1.0
   else
      timesinceequip = (updateframe-equipped2frame) + totalupdateelapsed
      canequip = timesinceequip > 1.0
   end

   if not canequip then
      return false
   end

   for i=0,lastCheck do
      if gCooldownTrinkets[i].bag ~= WEARING then
         cdstart, cdduration = GetContainerItemCooldown(gCooldownTrinkets[i].bag, gCooldownTrinkets[i].slot)
         if GetTime() > cdstart+cdduration-30 then
            PickupContainerItem(gCooldownTrinkets[i].bag, gCooldownTrinkets[i].slot)
            PickupInventoryItem(slot)
            
            gCooldownTrinkets[i].bag = WEARING
            gCooldownTrinkets[i].slot = 12 - slot

            updateframe = 0.0
            if slot == 13 then
               equipped1frame = updateframe
            else
               equipped2frame = updateframe
            end
            return true
         end
      end
   end

   return false
end

local function TryPassiveEquip(slot, trinket)
   for i=0,gNumPassiveTrinkets-1 do
      if gPassiveTrinkets[i].bag ~= WEARING and gPassiveTrinkets[i].name == trinket then
         PickupContainerItem(gPassiveTrinkets[i].bag, gPassiveTrinkets[i].slot)
         PickupInventoryItem(slot)
    
         gPassiveTrinkets[i].bag = WEARING
         gPassiveTrinkets[i].slot = 12 - slot

         updateframe = 0.0
         
         if slot == 13 then
            equipped1frame = updateframe
         else
            equipped2frame = updateframe
         end            
         return true
      end
   end

   return false
end

-- note this function assumes carrot is in passives list
local function EquipCarrot(slot)

   -- if it is already equipped return "success" so the equip logic doesn't run
   itemLink = GetInventoryItemLink("player", slot)
   _,_,trinket = string.find(itemLink or "","item:%d+.+%[(.+)%]")

   if trinket == "Carrot on a Stick" then      
      return true
   end

   -- return false for the other slot so normal equip logic will run
   itemLink = GetInventoryItemLink("player", 27-slot)
   _,_,trinket = string.find(itemLink or "","item:%d+.+%[(.+)%]")

   if trinket == "Carrot on a Stick" then
      return false
   end
   
   -- if cursor has an object in it, don't run the rest of the equip logic
   if CursorHasItem() then
      return true
   end

   if slot == 13 then
      timesinceequip = (updateframe-equipped1frame) + totalupdateelapsed
      canequip = timesinceequip > 1.0
   else
      timesinceequip = (updateframe-equipped2frame) + totalupdateelapsed
      canequip = timesinceequip > 1.0
   end

   if not canequip then
      return false
   end

   return TryPassiveEquip(slot, "Carrot on a Stick")   
end

function FlagEquipADTrinket(equip)
   gEquipADTrinket1 = equip
   gEquipADTrinket2 = equip
   ADEquippedTime = GetTime()
end

-- note this function assumes AD trinket is in passives list
local function EquipADTrinket(slot)

   -- if it is already equipped return "success" so the equip logic doesn't run
   itemLink = GetInventoryItemLink("player", slot)
   _,_,trinket = string.find(itemLink or "","item:%d+.+%[(.+)%]")

   if trinket == "Argent Dawn Commission" or
      trinket == "Seal of the Dawn" or
      trinket == "Rune of the Dawn" then
      ADAlreadyEquipped = true
      return true
    else
      ADAlreadyEquipped = false
    end

   -- return false for the other slot so normal equip logic will run
   itemLink = GetInventoryItemLink("player", 27-slot)
    _,_,trinket = string.find(itemLink or "","item:%d+.+%[(.+)%]")

    if trinket == "Argent Dawn Commission" or
      trinket == "Seal of the Dawn" or
      trinket == "Rune of the Dawn" then
      return false
    end
   
   -- if cursor has an object in it, don't run the rest of the equip logic
   if CursorHasItem() then
      return true
   end

   if slot == 13 then
      timesinceequip = (updateframe-equipped1frame) + totalupdateelapsed
      canequip = timesinceequip > 1.0
   else
      timesinceequip = (updateframe-equipped2frame) + totalupdateelapsed
      canequip = timesinceequip > 1.0
   end

   if not canequip then
      return false
   end

   ret = TryPassiveEquip(slot, "Seal of the Dawn")
   if not ret then
      ret = TryPassiveEquip(slot, "Rune of the Dawn")
   end
   if not ret then
      ret = TryPassiveEquip(slot, "Argent Dawn Commission")
   end

   return ret
end

local function MTPlayerDead()
   pdead = UnitIsDeadOrGhost("player")

	for i=1,16 do      
      icon = UnitBuff("player",i)

      -- no buffs left, return false
      if not icon then
         return pdead
      end

		if icon == "Interface\\Icons\\Ability_Rogue_FeignDeath" then
			pdead = false
         break
		end
	end

   return pdead
end

local function MTJustEnteredUndead()
   
   nowZone = GetRealZoneText()

   if nowZone ~= gLastZone and (nowZone == "Eastern Plaguelands" or nowZone == "Western Plaguelands" or
      nowZone == "Stratholme" or nowZone == "Scholomance") then
      return true
   else
      gEquipADTrinket = false
   end

   return false
end

gMountStarted = false
gMountStartTime = 0
gMountFinishTime = 0

local function MTIsOnMount()

	for i=1,16 do
      icon = UnitBuff("player",i)
      
      -- no buffs left, return false
      if not icon then
         return false
      end

      skipFirst = false

      if string.find(icon or "", "Tiger") then
         if gMountStarted then
            if gMountFinishTime >= (gMountStartTime+2.8) then
               gMountStarted = false
               return true
            end
         else
            skipFirst = true
         end
      end

      icon = string.sub(icon, 17)

      found = string.find(icon or "","Ability_Mount")

      if not skipFirst and found then
         return true
      end

      found = string.find(icon or "","INV_Misc_QirajiCrystal")

      if found then
         return true
      end

		if icon == "INV_Misc_Branch_01"     or         
         icon == "Spell_Nature_Swiftness" then
			return true
		end
	end

   return false
end

-- minimap button and options

function MTLoadOptions()
   MTOptionsCarrot:SetChecked(gOpts.AutoEquipCarrot)
   MTOptionsAD:SetChecked(gOpts.AskArgentDawn)
end

function MTSaveOptions()
   gOpts.AutoEquipCarrot = MTOptionsCarrot:GetChecked()
   gOpts.AskArgentDawn = MTOptionsAD:GetChecked()
end

function MTOptionsButtonInit()
   MTOptionsButton:SetPoint("CENTER", "Minimap", "CENTER", gCurrOptBtX, gCurrOptBtY)
end

function MTOptionsButtonUpdate(elapsed)
   if not gRightDragActive then
      return
   end

   currx, curry = GetCursorPosition()

   if currx == gCursorX and curry == gCursorY then
      return
   end

   dx = currx - gCursorX
   dy = curry - gCursorY

   gCursorX = currx
   gCursorY = curry

   mx, my = Minimap:GetCenter()
   btX, btY = MTOptionsButton:GetCenter()
   btX = btX - mx
   btY = btY - my

   dx = min(max(btX + dx, -gMinimapRadius), gMinimapRadius)
   dy = min(max(btY + dy, -gMinimapRadius), gMinimapRadius)

   ntheta = atan(dy/dx)
   if dx < 0 then
      ntheta = ntheta + 180
   end

   gCurrOptBtX = gMinimapRadius * cos(ntheta)
   gCurrOptBtY = gMinimapRadius * sin(ntheta)

   MTOptionsButton:SetPoint("CENTER", "Minimap", "CENTER", gCurrOptBtX, gCurrOptBtY)
end

function MTDialog(msg, func)
   gDialogFunc = func

   --message("dialog called")
   YesNoPrompt:SetText(msg)
   YesNoDialog:Show()
end

local function MTFixOlderVersions()
   if gSetupList[1].numPassives == 0 and gSetupList[1].numCooldowns == 0 then
      if gNumPassiveTrinkets > 0 or gNumCooldownTrinkets > 0 then
         MTTCopy(gSetupList[1].passiveList, gPassiveTrinkets)
         MTTCopy(gSetupList[1].cooldownList, gCooldownTrinkets)
         gSetupList[1].numPassives = gNumPassiveTrinkets
         gSetupList[1].numCooldowns = gNumCooldownTrinkets
         gNumPassiveTrinkets = 0
         gNumCooldownTrinkets = 0
         gPassiveTrinkets = {}
         gCooldownTrinkets = {}
         gCurrSetup = 1
      end
   end
end

function MTInit()
   this:RegisterEvent("PLAYER_LOGIN")

   SlashCmdList["MehTrinketeerHelp"] = MTSlashHelp
   SlashCmdList["MehTrinketeerShow"] = MTSlashShow
	SLASH_MehTrinketeerHelp1 = "/mehhelp";
	SLASH_MehTrinketeerShow1 = "/mehshow";

   DEFAULT_CHAT_FRAME:AddMessage("[MehTrinketeer] MehTrinketeer loaded successfully.", 225, 225, 0)
   DEFAULT_CHAT_FRAME:AddMessage("[MehTrinketeer] Use /mehshow to show/hide addon.  Use /mehhelp for quickstart.", 225, 225, 0)
end

function MTLoad()

   -- check tooltrinket slots now
   itemLink = GetInventoryItemLink("player", 13)
   _,_,currTrinket1ID,currTrinket1 = string.find(itemLink or "","item:(%d+).+%[(.+)%]")
   itemLink = GetInventoryItemLink("player", 14)
   _,_,currTrinket2ID,currTrinket2 = string.find(itemLink or "","item:(%d+).+%[(.+)%]")

   if currTrinket1ID then
      _,_,_,_,_,_,_,_,itemTexture = GetItemInfo(currTrinket1ID or "")
      MTEquipped1Icon:SetTexture(itemTexture)

      found, idx, listtype = MTFindInLists(currTrinket1)

      if not found then
         if not gPassiveTrinkets[gNumPassiveTrinkets] then
            gPassiveTrinkets[gNumPassiveTrinkets] = {}
         end
                
         gPassiveTrinkets[gNumPassiveTrinkets].bag = WEARING
         gPassiveTrinkets[gNumPassiveTrinkets].slot = TRINKETSLOT1
         gPassiveTrinkets[gNumPassiveTrinkets].name = currTrinket1
         gPassiveTrinkets[gNumPassiveTrinkets].texture = itemTexture
         gNumPassiveTrinkets = gNumPassiveTrinkets + 1
      else
         if listtype == "Passive" then
            -- make sure this is not run on the first update
            if gPassiveTrinkets[idx].bag ~= WEARING and (gLastPassiveHighlight >= 0 and gLastCooldownHighlight >= 0) then
               if not gAutoTrinketEquip1 then
                  gLock1 = true;
                  MTLock1:SetChecked(true)
                end

               gAutoTrinketEquip1 = false
            end            
            gPassiveTrinkets[idx].bag = WEARING
            gPassiveTrinkets[idx].slot = TRINKETSLOT1
            gPassiveTrinkets[idx].name = currTrinket1
            gPassiveTrinkets[idx].texture = itemTexture
         elseif listtype == "Cooldown" then
            -- make sure this is not run on the first update
            if gCooldownTrinkets[idx].bag ~= WEARING and (gLastPassiveHighlight >= 0 and gLastCooldownHighlight >= 0) then
               if not gAutoTrinketEquip1 then
                  gLock1 = true;
                  MTLock1:SetChecked(true)
                end

               gAutoTrinketEquip1 = false
            end   

            gCooldownTrinkets[idx].bag = WEARING
            gCooldownTrinkets[idx].slot = TRINKETSLOT1
            gCooldownTrinkets[idx].name = currTrinket1
            gCooldownTrinkets[idx].texture = itemTexture
         end
      end
   end

   if currTrinket2ID then
      _,_,_,_,_,_,_,_,itemTexture = GetItemInfo(currTrinket2ID or "")
      MTEquipped2Icon:SetTexture(itemTexture)

      found, idx, listtype = MTFindInLists(currTrinket2)

      if not found then
         if not gPassiveTrinkets[gNumPassiveTrinkets] then
            gPassiveTrinkets[gNumPassiveTrinkets] = {}
         end         
                    
         gPassiveTrinkets[gNumPassiveTrinkets].bag = WEARING
         gPassiveTrinkets[gNumPassiveTrinkets].slot = TRINKETSLOT2
         gPassiveTrinkets[gNumPassiveTrinkets].name = currTrinket2
         gPassiveTrinkets[gNumPassiveTrinkets].texture = itemTexture
         gNumPassiveTrinkets = gNumPassiveTrinkets + 1
      else
         if listtype == "Passive" then
            if gPassiveTrinkets[idx].bag ~= WEARING and (gLastPassiveHighlight >= 0 and gLastCooldownHighlight >= 0) then
               if not gAutoTrinketEquip2 then
                  gLock2 = true;
                  MTLock2:SetChecked(true)
                end

               gAutoTrinketEquip2 = false
            end 
            gPassiveTrinkets[idx].bag = WEARING
            gPassiveTrinkets[idx].slot = TRINKETSLOT2
            gPassiveTrinkets[idx].name = currTrinket2
            gPassiveTrinkets[idx].texture = itemTexture
         elseif listtype == "Cooldown" then
            if gCooldownTrinkets[idx].bag ~= WEARING and (gLastPassiveHighlight >= 0 and gLastCooldownHighlight >= 0) then
               if not gAutoTrinketEquip2 then
                  gLock2 = true;
                  MTLock2:SetChecked(true)
                end

               gAutoTrinketEquip2 = false
            end
            gCooldownTrinkets[idx].bag = WEARING
            gCooldownTrinkets[idx].slot = TRINKETSLOT2
            gCooldownTrinkets[idx].name = currTrinket2
            gCooldownTrinkets[idx].texture = itemTexture
         end
      end
   end

   -- if any bags are not equipped then exit now (this seems to be a wow quirk on startup)
   --for i = 0,4 do
   --   if GetContainerNumSlots(i) == 0 then
   --      return
   --   end
   --end
   
   -- build checklists
   passiveChecklist = {}
   cooldownChecklist = {}
   oldNumPassiveTrinkets = gNumPassiveTrinkets
   oldNumCooldownTrinkets = gNumCooldownTrinkets

   for i = 0,oldNumPassiveTrinkets-1 do
      if not passiveChecklist[i] then
         passiveChecklist[i] = {}
      end
      passiveChecklist[i].name = gPassiveTrinkets[i].name
      if currTrinket1 == passiveChecklist[i].name or currTrinket2 == passiveChecklist[i].name then
         passiveChecklist[i].present = true
      else
         passiveChecklist[i].present = false
      end
   end
   for i = 0,oldNumCooldownTrinkets-1 do
      if not cooldownChecklist[i] then
         cooldownChecklist[i] = {}
      end
      cooldownChecklist[i].name = gCooldownTrinkets[i].name
      if currTrinket1 == gCooldownTrinkets[i].name or currTrinket2 == gCooldownTrinkets[i].name then
         cooldownChecklist[i].present = true
      else
         cooldownChecklist[i].present = false
      end
   end

   -- any newly equipped tooltrinkets go into passive list   
   for i = 0,4 do
      for j = 1,GetContainerNumSlots(i) do
			itemLink = GetContainerItemLink(i,j)
		
			if itemLink then
				_,_,itemID,itemName = string.find(itemLink or "","item:(%d+).+%[(.+)%]")
				_,_,_,_,_,_,_,equipSlot,itemTexture = GetItemInfo(itemID or "")            
               
				if equipSlot=="INVTYPE_TRINKET" then
               -- flag that it has been found
               for k = 0,oldNumPassiveTrinkets-1 do
                  if itemName == passiveChecklist[k].name then
                     passiveChecklist[k].present = true
                  end
               end
               for k = 0,oldNumCooldownTrinkets-1 do
                  if itemName == cooldownChecklist[k].name then
                     cooldownChecklist[k].present = true
                  end
               end

               found, idx, listtype = MTFindInLists(itemName)
             
               if not found then
                  if not gPassiveTrinkets[gNumPassiveTrinkets] then
                     gPassiveTrinkets[gNumPassiveTrinkets] = {}
                  end
                
                     gPassiveTrinkets[gNumPassiveTrinkets].bag = i
                     gPassiveTrinkets[gNumPassiveTrinkets].slot = j
                     gPassiveTrinkets[gNumPassiveTrinkets].name = itemName
                     gPassiveTrinkets[gNumPassiveTrinkets].texture = itemTexture
                     gNumPassiveTrinkets = gNumPassiveTrinkets + 1
               else
                  if listtype == "Passive" then
                     gPassiveTrinkets[idx].bag = i
                     gPassiveTrinkets[idx].slot = j
                  elseif listtype == "Cooldown" then
                     gCooldownTrinkets[idx].bag = i
                     gCooldownTrinkets[idx].slot = j
                  end                  
               end
				end
			end
		end
	end

   -- remove anything that was destroyed or put back in bank
   for i = 0,oldNumPassiveTrinkets-1 do
      if passiveChecklist[i].present == false then
         if (passiveChecklist[i].name) then
            MTRemoveFromList(gPassiveTrinkets, gNumPassiveTrinkets, i)            
            gNumPassiveTrinkets = gNumPassiveTrinkets - 1
 
            -- if empty, hide last slot
            if gNumPassiveTrinkets == 0 then
               MTPassive1:Hide()
            end

            if gLastPassiveHighlight > 0 then
               getglobal("MTPassive"..gLastPassiveHighlight):SetChecked(false)
            end
         end
      end
   end
   for i = 0,oldNumCooldownTrinkets-1 do
      if cooldownChecklist[i].present == false then
         if (cooldownChecklist[i].name) then
            MTRemoveFromList(gCooldownTrinkets, gNumCooldownTrinkets, i)            
            gNumCooldownTrinkets = gNumCooldownTrinkets - 1
 
            -- if empty, hide last slot
            if gNumCooldownTrinkets == 0 then
               MTCooldown1:Hide()
            end

            if gLastCooldownHighlight > 0 then
               getglobal("MTCooldown"..gLastCooldownHighlight):SetChecked(false)
            end
         end
      end
   end

   -- set scale now
   MTTrinkets:SetScale(gButtonsScale)

   -- arrange passives
   numRows = math.ceil(math.min(gNumPassiveTrinkets,gMaxNumCols*gMaxNumRows) / gMaxNumCols)
   --if (gNumPassiveTrinkets < 5) then
   --   numCols = math.mod(math.min(gNumPassiveTrinkets,gMaxNumCols*gMaxNumRows), gMaxNumCols)
   --else
      numCols = gMaxNumCols
   --end      

   local i, j

   for j = 1,numRows do
      for i = 1,numCols do
         if ((j-1)*numCols+i) > gNumPassiveTrinkets then
            break;
         end
         currTrinket = getglobal("MTPassive"..((j-1)*numCols+i))
         getglobal("MTPassive"..((j-1)*numCols+i).."Icon"):SetTexture(gPassiveTrinkets[((j-1)*numCols+i-1)].texture)
         currTrinket:SetPoint("TOPLEFT","MTFrameTrinkets","TOPLEFT", gPassivesXstart*1/gButtonsScale + gButtonNext*(i-1), 
                              gPassivesYstart*1/gButtonsScale - gButtonNext*(j-1))
      end
   end

   -- queue arrows
   local qarrowsXStart = 8 + gMaxNumCols * gButtonNext * gButtonsScale + 40   -- 16 + 8 + 16
   local qarrowsYStart = -50;

   MTQueueLeft:SetPoint("CENTER","MTFrameTrinkets","TOPLEFT", qarrowsXStart - 16, qarrowsYStart)
   MTQueueUp:SetPoint("CENTER","MTFrameTrinkets","TOPLEFT", qarrowsXStart, qarrowsYStart + 16)
   MTQueueRight:SetPoint("CENTER","MTFrameTrinkets","TOPLEFT", qarrowsXStart + 16, qarrowsYStart)
   MTQueueDown:SetPoint("CENTER","MTFrameTrinkets","TOPLEFT", qarrowsXStart, qarrowsYStart - 16)
   
   lblQueueArrows:SetPoint("CENTER","MTFrameTrinkets","TOPLEFT", qarrowsXStart, -10)

   qarrowsYStart = qarrowsYStart - 44  -- 8 + 16 + 20
   lblSendArrows:SetPoint("CENTER","MTFrameTrinkets","TOPLEFT", qarrowsXStart, qarrowsYStart)

   qarrowsYStart = qarrowsYStart - 28  -- 12 + 16
   MTTypeRight:SetPoint("CENTER","MTFrameTrinkets","TOPLEFT", qarrowsXStart, qarrowsYStart + 10)
   MTTypeLeft:SetPoint("CENTER","MTFrameTrinkets","TOPLEFT", qarrowsXStart, qarrowsYStart - 10)

   -- arrange cooldowns
   numRows = math.ceil(math.min(gNumCooldownTrinkets,gMaxNumCols*gMaxNumRows) / gMaxNumCols)
   --if (gNumPassiveTrinkets < 5) then
   --   numCols = math.mod(math.min(gNumCooldownTrinkets,gMaxNumCols*gMaxNumRows), gMaxNumCols)
   --else
      numCols = gMaxNumCols
   --end

   gCooldownsXstart = qarrowsXStart + 40 -- 8 + 16 + 16

   for j = 1,numRows do
      for i = 1,numCols do
         if ((j-1)*numCols+i) > gNumCooldownTrinkets then
            break;
         end
         currTrinket = getglobal("MTCooldown"..((j-1)*numCols+i))
         getglobal("MTCooldown"..((j-1)*numCols+i).."Icon"):SetTexture(gCooldownTrinkets[((j-1)*numCols+i-1)].texture)
         currTrinket:SetPoint("TOPLEFT","MTFrameTrinkets","TOPLEFT", gCooldownsXstart*1/gButtonsScale + gButtonNext*(i-1), 
                              gCooldownsYstart*1/gButtonsScale - gButtonNext*(j-1))
      end
   end

   -- the following is = 8 + numCols * gButtonNext * gButtonsScale + 16 + 16 * 3 + 16 + numCols * gButtonNext * gButtonScale + 8
   local tooltrinketsWidth = 2 * numCols * gButtonNext * gButtonsScale + 96
   -- the following is = 35 + numRows * gButtonNext * gButtonsScale + 25
   if gAdvancedOpen then
      advanced = gAdvancedHeight
   else
      advanced = 0
   end
   local tooltrinketsHeight = math.max(60 + numRows * gButtonNext * gButtonsScale + advanced, gTrinketsMinHeight + advanced)

   MTFrameTrinkets:SetWidth(tooltrinketsWidth)
   MTFrameTrinkets:SetHeight(tooltrinketsHeight)
 
   MTCooldownOn:SetPoint("LEFT", "MTFrameTrinkets", "TOPRIGHT", -132, -27)
   lblCooldownOn:SetPoint("LEFT", "MTCooldownOn", "RIGHT", 0, 0)

   -- draw selectors 1st frame only
   if gLastPassiveHighlight < 0 or gLastCooldownHighlight < 0 then      

      -- might as well do other first frame only stuff here
      -- sync checkboxes
      MTPassiveOn:SetChecked(gPassivesOn)
      MTCooldownOn:SetChecked(gCooldownsOn)

      -- make cooldowns visible
      --MTEquipped1Cooldown:Show()
      --MTEquipped2Cooldown:Show()

      if gNumPassiveTrinkets == 0 then
         MTCooldown1:SetChecked(true)
         gLastPassiveHighlight = 0
         gLastCooldownHighlight = 1
      else
         MTPassive1:SetChecked(true)
         gLastPassiveHighlight = 1
         gLastCooldownHighlight = 0
      end
   end

   -- make the palette

   MTPaletteTrinkets:SetScale(gButtonsScale)

   numRows = math.ceil(math.min(gNumPassiveTrinkets,gMaxNumCols*gMaxNumRows) / gMaxNumCols)
   currY = gPaletteYstart

   local i, j

   for j = 1,numRows do
      for i = 1,numCols do
         if ((j-1)*numCols+i) > gNumPassiveTrinkets then
            break;
         end
         currTrinket = getglobal("MTPassivePalette"..((j-1)*numCols+i))
         getglobal("MTPassivePalette"..((j-1)*numCols+i).."Icon"):SetTexture(gPassiveTrinkets[((j-1)*numCols+i-1)].texture)
         currTrinket:SetPoint("TOPLEFT","MTFramePalette","TOPLEFT", gPaletteXstart*1/gButtonsScale + gButtonNext*(i-1), 
                              currY*1/gButtonsScale - gButtonNext*(j-1))
      end
   end

   currY = currY - 5 - gButtonNext*numRows*gButtonsScale

   lblCooldowns:SetPoint("TOP", "MTFramePalette", "TOP", 0, currY)

   currY = currY - 15

   numRows = math.ceil(math.min(gNumCooldownTrinkets,gMaxNumCols*gMaxNumRows) / gMaxNumCols)

   for j = 1,numRows do
      for i = 1,numCols do
         if ((j-1)*numCols+i) > gNumCooldownTrinkets then
            break;
         end

         currTrinket = getglobal("MTCooldownPalette"..((j-1)*numCols+i))
         getglobal("MTCooldownPalette"..((j-1)*numCols+i).."Icon"):SetTexture(gCooldownTrinkets[((j-1)*numCols+i-1)].texture)
         currTrinket:SetPoint("TOPLEFT","MTFramePalette","TOPLEFT", gPaletteXstart*1/gButtonsScale + gButtonNext*(i-1), 
                              currY*1/gButtonsScale - gButtonNext*(j-1))                              
      end
   end


   tooltrinketsWidth = numCols * gButtonNext * gButtonsScale + 8 + 8*gButtonsScale
   tooltrinketsHeight = gPaletteYstart - currY + gButtonNext*numRows*gButtonsScale + 26

   MTFramePalette:SetWidth(tooltrinketsWidth)
   MTFramePalette:SetHeight(tooltrinketsHeight)

end

function MTShow()
   MTFrameEquipped:Show()

   -- show equipped

   itemLink = GetInventoryItemLink("player", 13)
   _,_,trinketid = string.find(itemLink or "","item:(%d+).+%[(.+)%]")

   if trinketid then
      MTEquipped1Icon:Show()
   else
      MTEquipped1Icon:Hide()
   end

   itemLink = GetInventoryItemLink("player", 14)
   _,_,trinketid = string.find(itemLink or "","item:(%d+).+%[(.+)%]")

   if trinketid then
      MTEquipped2Icon:Show()
   else
      MTEquipped2Icon:Hide()
   end

   

   -- show passives
   numRows = math.ceil(math.min(gNumPassiveTrinkets,gMaxNumCols*gMaxNumRows) / gMaxNumCols)
   -- go one more row just in case a type switch just happened
   numRows = numRows + 1
   for j = 1,numRows do
      for i = 1,gMaxNumCols do
         if ((j-1)*gMaxNumCols+i) > gNumPassiveTrinkets then
            getglobal("MTPassive"..((j-1)*gMaxNumCols+i)):Hide()
         else
            getglobal("MTPassive"..((j-1)*gMaxNumCols+i)):Show()
            getglobal("MTPassive"..((j-1)*gMaxNumCols+i).."Cooldown"):Show()
         end           
      end
   end

   -- palette passives   
   for j = 1,numRows do
      for i = 1,gMaxNumCols do
         if ((j-1)*gMaxNumCols+i) > gNumPassiveTrinkets then
            getglobal("MTPassivePalette"..((j-1)*gMaxNumCols+i)):Hide()
         else
            getglobal("MTPassivePalette"..((j-1)*gMaxNumCols+i)):Show()
            getglobal("MTPassivePalette"..((j-1)*gMaxNumCols+i).."Cooldown"):Show()
         end           
      end
   end

   -- show cooldowns
   numRows = math.ceil(math.min(gNumCooldownTrinkets,gMaxNumCols*gMaxNumRows) / gMaxNumCols)
   -- go one more row just in case a type switch just happened
   numRows = numRows + 1
   for j = 1,numRows do
      for i = 1,gMaxNumCols do
         if ((j-1)*gMaxNumCols+i) > gNumCooldownTrinkets then
            getglobal("MTCooldown"..((j-1)*gMaxNumCols+i)):Hide()
         else
            getglobal("MTCooldown"..((j-1)*gMaxNumCols+i)):Show()
            getglobal("MTCooldown"..((j-1)*gMaxNumCols+i).."Cooldown"):Show()
         end           
      end
   end

   -- palette cooldowns
   for j = 1,numRows do
      for i = 1,gMaxNumCols do
         if ((j-1)*gMaxNumCols+i) > gNumCooldownTrinkets then
            getglobal("MTCooldownPalette"..((j-1)*gMaxNumCols+i)):Hide()
         else
            getglobal("MTCooldownPalette"..((j-1)*gMaxNumCols+i)):Show()
            getglobal("MTCooldownPalette"..((j-1)*gMaxNumCols+i).."Cooldown"):Show()
         end           
      end
   end
end

gUseTimer1 = 0
gUseTimer2 = 0

function MTEquippedSlotClick(button)
   
   this:SetChecked(false)

   if button=="RightButton" and not MerchantFrame:IsVisible() then
      _,_,bnum = string.find(this:GetName(),"MTEquipped(%d+)")
      bnum = tonumber(bnum)-1

      -- don't check cooldown so user gets error msg
      --_, cdduration = GetInventoryItemCooldown("player",13+bnum)
      --if cdduration == 0 then
         UseInventoryItem(13+bnum)
         if bnum == 0 then
            gUseTimer1 = 0
         else
            gUseTimer2 = 0
         end
      --end
   end
end

-- highlight and (deprecated) manual trinket equip

function MTListSlotClick(button)
   _,_,typ, bnum = string.find(this:GetName(),"MT(%a+)(%d+)")

   bnum = tonumber(bnum)

   if typ == "Passive" then
      if bnum == gLastPassiveHighlight then
         -- keep it highlighted
         getglobal("MTPassive"..gLastPassiveHighlight):SetChecked(true)
         return
      end

      if gLastPassiveHighlight > 0 then
         getglobal("MTPassive"..gLastPassiveHighlight):SetChecked(false)
      end
      if gLastCooldownHighlight > 0 then
         getglobal("MTCooldown"..gLastCooldownHighlight):SetChecked(false)
      end

      gLastPassiveHighlight = bnum
      gLastCooldownHighlight = 0

      --if not gPassivesOn then
         -- manual equip
      --   PickupContainerItem(gPassiveTrinkets[bnum-1].bag, gPassiveTrinkets[bnum-1].slot)
         
      --   if button == "LeftButton" then
      --      PickupInventoryItem(13)
      --   else
      --      PickupInventoryItem(14)
      --   end
      --end
   else
      if bnum == gLastCooldownHighlight then
         -- keep it highlighted
         getglobal("MTCooldown"..gLastCooldownHighlight):SetChecked(true)
         return
      end

      if gLastPassiveHighlight > 0 then
         getglobal("MTPassive"..gLastPassiveHighlight):SetChecked(false)
      end
      if gLastCooldownHighlight > 0 then
         getglobal("MTCooldown"..gLastCooldownHighlight):SetChecked(false)
      end

      gLastPassiveHighlight = 0
      gLastCooldownHighlight = bnum

      --if not gCooldownsOn then
         -- manual equip
      --   PickupContainerItem(gCooldownTrinkets[bnum-1].bag, gCooldownTrinkets[bnum-1].slot)
         
      --   if button == "LeftButton" then
      --      PickupInventoryItem(13)
      --   else
      --      PickupInventoryItem(14)
      --   end         
      --end
   end
end

local function MTGetEquippedName(slot)
   itemLink = GetInventoryItemLink("player", slot)
    _,_,name = string.find(itemLink or "","item:%d+.+%[(.+)%]")
    return name
end

local function MTDoNothing()

end

function MTPaletteSlotClick(button)
   this:SetChecked(false)

   -- if cursor has an object in it, don't mess with it and return failure
   if CursorHasItem() then
      return false
   end
  
   _,_,typ, bnum = string.find(this:GetName(),"MT(%a+)(%d+)")

   bnum = tonumber(bnum)-1

   if typ == "PassivePalette" then
      if gPassiveTrinkets[bnum].slot == TRINKETSLOT1 then
         PickupInventoryItem(13)
      elseif gPassiveTrinkets[bnum].slot == TRINKETSLOT2 then
         PickupInventoryItem(14)
      else
         PickupContainerItem(gPassiveTrinkets[bnum].bag, gPassiveTrinkets[bnum].slot)
      end
      
      if button == "LeftButton" then
         PickupInventoryItem(13)
      else
         PickupInventoryItem(14)
      end         
   else
      if gCooldownTrinkets[bnum].slot == TRINKETSLOT1 then
         PickupInventoryItem(13)
      elseif gCooldownTrinkets[bnum].slot == TRINKETSLOT2 then
         PickupInventoryItem(14)
      else
         PickupContainerItem(gCooldownTrinkets[bnum].bag, gCooldownTrinkets[bnum].slot)
      end
      
      if button == "LeftButton" then
         PickupInventoryItem(13)
      else
         PickupInventoryItem(14)
      end         
   end

   --if button == "LeftButton" then
   --   gAutoTrinketEquip1 = false
   --else
   --   gAutoTrinketEquip2 = false
   --end

   -- hijack update
   oldfunc = MTUpdate
   MTUpdate = MTDoNothing
   MTLoad()
   MTShow()
   MTUpdate = oldfunc      
end

function MTLockChkClick()
   if this:GetID() == 1 then
      gLock1 = not gLock1
      MTLock1:SetChecked(gLock1)
      if gLock1 then
         gEquipADTrinket1 = false
      end
   else
      gLock2 = not gLock2
      MTLock2:SetChecked(gLock2)
      if gLock2 then
         gEquipADTrinket2 = false
      end
   end
end

function MTTrinketChkClick()
   if this:GetID() == 1 then
      gPassivesOn = not gPassivesOn
      MTPassiveOn:SetChecked(gPassivesOn)
   else
      gCooldownsOn = not gCooldownsOn
      MTCooldownOn:SetChecked(gCooldownsOn)
   end
end
      

-- change priorities

local function MTSwapCooldowns(trink1, trink2)
   temp = {}

   temp.bag       = gCooldownTrinkets[trink1].bag   
   temp.slot      = gCooldownTrinkets[trink1].slot  
   temp.name      = gCooldownTrinkets[trink1].name      
   temp.texture   = gCooldownTrinkets[trink1].texture

   gCooldownTrinkets[trink1].bag       = gCooldownTrinkets[trink2].bag
   gCooldownTrinkets[trink1].slot      = gCooldownTrinkets[trink2].slot
   gCooldownTrinkets[trink1].name      = gCooldownTrinkets[trink2].name
   gCooldownTrinkets[trink1].texture   = gCooldownTrinkets[trink2].texture

   gCooldownTrinkets[trink2].bag       = temp.bag
   gCooldownTrinkets[trink2].slot      = temp.slot
   gCooldownTrinkets[trink2].name      = temp.name
   gCooldownTrinkets[trink2].texture   = temp.texture
end

function MTSwapPassives(trink1, trink2)
   temp = {}

   temp.bag       = gPassiveTrinkets[trink1].bag   
   temp.slot      = gPassiveTrinkets[trink1].slot  
   temp.name      = gPassiveTrinkets[trink1].name      
   temp.texture   = gPassiveTrinkets[trink1].texture

   gPassiveTrinkets[trink1].bag       = gPassiveTrinkets[trink2].bag
   gPassiveTrinkets[trink1].slot      = gPassiveTrinkets[trink2].slot
   gPassiveTrinkets[trink1].name      = gPassiveTrinkets[trink2].name
   gPassiveTrinkets[trink1].texture   = gPassiveTrinkets[trink2].texture

   gPassiveTrinkets[trink2].bag       = temp.bag
   gPassiveTrinkets[trink2].slot      = temp.slot
   gPassiveTrinkets[trink2].name      = temp.name
   gPassiveTrinkets[trink2].texture   = temp.texture
end

function MTChgPriorityLeft()
   if gLastPassiveHighlight == 0 then
      -- already has top priority
      if gLastCooldownHighlight == 1 then
         return
      end
      MTSwapCooldowns(gLastCooldownHighlight-1, gLastCooldownHighlight-2)

      getglobal("MTCooldown"..gLastCooldownHighlight):SetChecked(false)
      getglobal("MTCooldown"..gLastCooldownHighlight-1):SetChecked(true)
      gLastCooldownHighlight = gLastCooldownHighlight-1
   else
      -- already has top priority
      if gLastPassiveHighlight == 1 then
         return
      end
      MTSwapPassives(gLastPassiveHighlight-1, gLastPassiveHighlight-2)

      getglobal("MTPassive"..gLastPassiveHighlight):SetChecked(false)
      getglobal("MTPassive"..gLastPassiveHighlight-1):SetChecked(true)
      gLastPassiveHighlight = gLastPassiveHighlight-1
   end

   MTLoad()
   MTShow()
end

function MTChgPriorityUp()
   if gLastPassiveHighlight == 0 then
      -- already has top priority
      if gLastCooldownHighlight == 1 then
         return
      end
      destination = math.max(gLastCooldownHighlight-5, 1)
      for i=destination-1, gLastCooldownHighlight-2 do
         MTSwapCooldowns(gLastCooldownHighlight-1-i, gLastCooldownHighlight-2-i)
      end

      getglobal("MTCooldown"..gLastCooldownHighlight):SetChecked(false)
      getglobal("MTCooldown"..destination):SetChecked(true)
      gLastCooldownHighlight = destination
   else
      -- already has top priority
      if gLastPassiveHighlight == 1 then
         return
      end
      destination = math.max(gLastPassiveHighlight-5, 1)
      for i=destination-1, gLastPassiveHighlight-2 do
         MTSwapPassives(gLastPassiveHighlight-1-i, gLastPassiveHighlight-2-i)
      end

      getglobal("MTPassive"..gLastPassiveHighlight):SetChecked(false)
      getglobal("MTPassive"..destination):SetChecked(true)
      gLastPassiveHighlight = destination
   end

   MTLoad()
   MTShow()
end

function MTChgPriorityRight()
   if gLastPassiveHighlight == 0 then
      -- already has lowest priority
      if gLastCooldownHighlight == gNumCooldownTrinkets then
         return
      end
      MTSwapCooldowns(gLastCooldownHighlight-1, gLastCooldownHighlight)

      getglobal("MTCooldown"..gLastCooldownHighlight):SetChecked(false)
      getglobal("MTCooldown"..gLastCooldownHighlight+1):SetChecked(true)
      gLastCooldownHighlight = gLastCooldownHighlight+1
   else
      -- already has lowest priority
      if gLastPassiveHighlight == gNumPassiveTrinkets then
         return
      end
      MTSwapPassives(gLastPassiveHighlight-1, gLastPassiveHighlight)

      getglobal("MTPassive"..gLastPassiveHighlight):SetChecked(false)
      getglobal("MTPassive"..gLastPassiveHighlight+1):SetChecked(true)
      gLastPassiveHighlight = gLastPassiveHighlight+1
   end

   MTLoad()
   MTShow()
end

function MTChgPriorityDown()
   if gLastPassiveHighlight == 0 then
      -- already has lowest priority
      if gLastCooldownHighlight == gNumCooldownTrinkets then
         return
      end
      destination = math.min(gLastCooldownHighlight+5, gNumCooldownTrinkets)
      for i=gLastCooldownHighlight-1, destination-2 do
         MTSwapCooldowns(i, i+1)
      end

      getglobal("MTCooldown"..gLastCooldownHighlight):SetChecked(false)
      getglobal("MTCooldown"..destination):SetChecked(true)
      gLastCooldownHighlight = destination
   else
      -- already has lowest priority
      if gLastPassiveHighlight == gNumPassiveTrinkets then
         return
      end
      destination = math.min(gLastPassiveHighlight+5, gNumPassiveTrinkets)
      for i=gLastPassiveHighlight-1, destination-2 do
         MTSwapPassives(i, i+1)
      end

      getglobal("MTPassive"..gLastPassiveHighlight):SetChecked(false)
      getglobal("MTPassive"..destination):SetChecked(true)
      gLastPassiveHighlight = destination
   end

   MTLoad()
   MTShow()
end

-- transfer lists

function MTXPassiveToCooldown()
   if gNumPassiveTrinkets == 0 or gLastPassiveHighlight == 0 then
      return
   end
   
   tooltrinket = MTRemoveFromList(gPassiveTrinkets, gNumPassiveTrinkets, gLastPassiveHighlight-1)
   gNumPassiveTrinkets = gNumPassiveTrinkets - 1
 
   -- if empty, hide last slot
   if gNumPassiveTrinkets == 0 then
      MTPassive1:Hide()
      MTPassivePalette1:Hide()
   end

   if not gCooldownTrinkets[gNumCooldownTrinkets] then
      gCooldownTrinkets[gNumCooldownTrinkets] = {}
   end

   -- force copy by value cuz lua sucks
   gCooldownTrinkets[gNumCooldownTrinkets].bag        = tooltrinket.bag    
   gCooldownTrinkets[gNumCooldownTrinkets].slot       = tooltrinket.slot   
   gCooldownTrinkets[gNumCooldownTrinkets].name       = tooltrinket.name   
   gCooldownTrinkets[gNumCooldownTrinkets].texture    = tooltrinket.texture
   gNumCooldownTrinkets = gNumCooldownTrinkets + 1

   getglobal("MTPassive"..gLastPassiveHighlight):SetChecked(false)
   getglobal("MTCooldown"..gNumCooldownTrinkets):SetChecked(true)

   gLastCooldownHighlight = gNumCooldownTrinkets
   gLastPassiveHighlight = 0

   -- update display
   MTLoad()
   MTShow()
end

function MTXCooldownToPassive()
   if gNumCooldownTrinkets == 0 or gLastCooldownHighlight == 0 then
      return
   end

   tooltrinket = MTRemoveFromList(gCooldownTrinkets, gNumCooldownTrinkets, gLastCooldownHighlight-1)
   gNumCooldownTrinkets = gNumCooldownTrinkets - 1

   -- if empty, hide last slot
   if gNumCooldownTrinkets == 0 then
      MTCooldown1:Hide()
      MTCooldownPalette1:Hide()
   end
 
   if not gPassiveTrinkets[gNumPassiveTrinkets] then
      gPassiveTrinkets[gNumPassiveTrinkets] = {}
   end

   gPassiveTrinkets[gNumPassiveTrinkets].bag        = tooltrinket.bag    
   gPassiveTrinkets[gNumPassiveTrinkets].slot       = tooltrinket.slot   
   gPassiveTrinkets[gNumPassiveTrinkets].name       = tooltrinket.name   
   gPassiveTrinkets[gNumPassiveTrinkets].texture    = tooltrinket.texture
   gNumPassiveTrinkets = gNumPassiveTrinkets + 1

   getglobal("MTCooldown"..gLastCooldownHighlight):SetChecked(false)
   getglobal("MTPassive"..gNumPassiveTrinkets):SetChecked(true)

   gLastCooldownHighlight = 0
   gLastPassiveHighlight = gNumPassiveTrinkets

   -- update display
   MTLoad()
   MTShow()
end
     
-- moving window around
function MTMoveOnMouseUp(arg1, frame)
	if arg1=="LeftButton" then
		frame:StopMovingOrSizing()
   end
end
function MTMoveOnMouseDown(arg1, frame)
   if arg1=="LeftButton" then
		frame:StartMoving()
	end
end

-- tooltip

-- deprecated
--function MTShowChkTooltip()

--   if this:GetRight()+200 < GetScreenWidth() then
--      GameTooltip:SetOwner(this,"ANCHOR_BOTTOMRIGHT")
--	else
--	   GameTooltip:SetOwner(this,"ANCHOR_BOTTOMLEFT")
--   end

--   GameTooltip:AddLine("If this is unchecked, you can right/left")
--   GameTooltip:AddLine("click to equip trinkets yourself.")

--   GameTooltip:Show()
--end

function MTShowLockTooltip()
   GameTooltip:SetOwner(this,"ANCHOR_BOTTOMRIGHT")
   GameTooltip:AddLine("Locked?")
   GameTooltip:Show()   
end

totaltooltipelapsed = 0

function MTUpdateTooltip(elapsed)
   if not GameTooltip:IsOwned(this) then
      return
   end

   totaltooltipelapsed = totaltooltipelapsed + elapsed

   if totaltooltipelapsed > TOOLTIP_UPDATE_TIME then
      MTShowTooltip()
      totaltooltipelapsed = 0
   end
end

function MTShowTooltip()
   if not this:GetRight() then
      return;
   end

   _,_,btyp,bnum = string.find(this:GetName(),"MT(%a+)(%d+)")

   bnum = tonumber(bnum)

   local tooltrinket = nil

   if btyp == "Passive" or btyp == "PassivePalette" then
      if gPassiveTrinkets[bnum-1].bag == WEARING then
         bnum = gPassiveTrinkets[bnum-1].slot * -1
      else
         tooltrinket = gPassiveTrinkets[bnum-1]
      end
      cmpscale = gButtonsScale
   elseif btyp == "Cooldown" or btyp == "CooldownPalette" then
      if gCooldownTrinkets[bnum-1].bag == WEARING then
         bnum = gCooldownTrinkets[bnum-1].slot * -1
      else
         tooltrinket = gCooldownTrinkets[bnum-1]
      end
      cmpscale = gButtonsScale
   elseif btyp == "Equipped" then      
      if bnum == 1 then
         itemLink = GetInventoryItemLink("player", 13)
         _,_,trinketid = string.find(itemLink or "","item:(%d+).+%[(.+)%]")
      else
         itemLink = GetInventoryItemLink("player", 14)
         _,_,trinketid = string.find(itemLink or "","item:(%d+).+%[(.+)%]")
      end
      cmpscale = 1.0
   end

   if this:GetRight()*cmpscale+300 < GetScreenWidth() then
      GameTooltip:SetOwner(this,"ANCHOR_BOTTOMRIGHT")
	else
	   GameTooltip:SetOwner(this,"ANCHOR_BOTTOMLEFT")
   end
   --GameTooltip:SetOwner(this,"ANCHOR_CURSOR")

   if not tooltrinket then      
      if bnum == 1 then
         GameTooltip:SetInventoryItem("player", 13)
      else
         GameTooltip:SetInventoryItem("player", 14)
      end
   else     
      GameTooltip:SetBagItem(tooltrinket.bag, tooltrinket.slot)
   end

   GameTooltip:Show()
end

-- reset lists
function MTReset()
   for i=1,gNumCooldownTrinkets do
      getglobal("MTCooldown"..i):Hide()
   end
   
   gNumPassiveTrinkets = 0
   gNumCooldownTrinkets = 0

   MTLoad()
   MTShow()
end

local function MTUpdateAdvanced()
   if gAdvancedOpen then
      MTAdvanced:SetText("Advanced <<")
      Advanced:Show()      
   else
      MTAdvanced:SetText("Advanced >>")
      Advanced:Hide()
   end

   MTLoad()
   MTShow()
end

function MTToggleAdvanced()
   gAdvancedOpen = not gAdvancedOpen
   MTUpdateAdvanced()
end

local function MTSaveCurrentSetup()
   gSetupList[gCurrSetup].passiveList = gPassiveTrinkets
   gSetupList[gCurrSetup].numPassives = gNumPassiveTrinkets
   gSetupList[gCurrSetup].cooldownList = gCooldownTrinkets
   gSetupList[gCurrSetup].numCooldowns = gNumCooldownTrinkets
end

local function MTSetCurrentSetup()
   gPassiveTrinkets = gSetupList[gCurrSetup].passiveList
   gNumPassiveTrinkets = gSetupList[gCurrSetup].numPassives
   gCooldownTrinkets = gSetupList[gCurrSetup].cooldownList
   gNumCooldownTrinkets = gSetupList[gCurrSetup].numCooldowns

   -- prevent locking
   gAutoTrinketEquip1 = true
   gAutoTrinketEquip2 = true

   MTLoad()
   MTShow()
end

function MTDeleteSetup()
   if gCurrSetup == 1 then
      return
   end

   tremove(gSetupList, gCurrSetup)
   gCurrSetup = 1
   MTSetCurrentSetup()
   MTSetupDropDownInit()
   MTContinentDropDownInit()
   MTZoneDropDownInit()
end

gSetupDDElapsed = 0

function MTSetupDropDownUpdate(elapsed)
   
   gSetupDDElapsed = gSetupDDElapsed + elapsed

   if gSetupDDElapsed > 15.0 then
      CloseDropDownMenus()
   end
end

function MTToggleDropDownMenu(arg1, arg2, arg3, arg4, arg5, arg6)

   gOldToggleDropDownMenu(arg1, arg2, arg3, arg4, arg5, arg6)

   if not MTCurrentSetupDropDown:IsVisible() then
      return
   end

   gSetupDDElapsed = 0
end

function MTSetupDDEnter(arg1)

   gOldUIDropDownMenu_StopCounting(arg1)

   if not MTCurrentSetupDropDown:IsVisible() then
      return
   end

   gSetupDDElapsed = -1000
end

function MTSetupDDLeave(arg1)

   gOldUIDropDownMenu_StartCounting(arg1)

   if not MTCurrentSetupDropDown:IsVisible() then
      return
   end

   gSetupDDElapsed = 0
end

------------------- Setup drop down

function MTAddNewSetup()
   newsetup = MTNewSetupText:GetText()
   
   if newsetup ~= "" then

      tinsert(gSetupList, {
                              name = newsetup,
                              passiveList = {},
                              numPassives = 0,
                              cooldownList = {},
                              numCooldowns = 0,
                              autoload = 1,
                              zone = 0,
                              zonename = ""
                           })
      MTSaveCurrentSetup()
      gCurrSetup = getn(gSetupList)
      MTSetCurrentSetup()
      MTSetupDropDownInit()
      MTContinentDropDownInit()
      MTZoneDropDownInit()
   end
end

function MTSetupDropDownOnClick()
   UIDropDownMenu_SetSelectedID(MTCurrentSetupDropDown, this:GetID());
   MTSaveCurrentSetup()
   gCurrSetup = MTCurrentSetupDropDown.selectedID         
   MTSetCurrentSetup()
   MTContinentDropDownInit()
   MTZoneDropDownInit()
end

gOldToggleDropDownMenu = 0
gOldUIDropDownMenu_StopCounting = 0
gOldUIDropDownMenu_StartCounting = 0

function MTSetupDropDownInit()
   local info

   UIDropDownMenu_ClearAll(MTCurrentSetupDropDown)
   DropDownList1.numButtons = 0

   for i=1,getn(gSetupList) do 
      info = {}
      info.text = gSetupList[i].name
      info.func = MTSetupDropDownOnClick
      UIDropDownMenu_AddButton(info)
   end

   UIDropDownMenu_SetSelectedID(MTCurrentSetupDropDown, gCurrSetup, 1);
end

function MTLoadSetupDropDown()
   UIDropDownMenu_Initialize(MTCurrentSetupDropDown, MTSetupDropDownInit)
   UIDropDownMenu_SetWidth(130, MTCurrentSetupDropDown)
   
   gOldToggleDropDownMenu = ToggleDropDownMenu
   gOldUIDropDownMenu_StopCounting = UIDropDownMenu_StopCounting
   gOldUIDropDownMenu_StartCounting = UIDropDownMenu_StartCounting

   ToggleDropDownMenu = MTToggleDropDownMenu
   UIDropDownMenu_StopCounting = MTSetupDDEnter
   UIDropDownMenu_StartCounting = MTSetupDDLeave
end

------------------- Continent drop down

local function MTGetZones(...)
   zones = {}

   for i=1,arg.n do
      tinsert(zones, arg[i])
   end

   return zones
end

local function MTGetInstances()
   instances = {}

   tinsert(instances, "Ahn'Qiraj")
   tinsert(instances, "Blackfathom Deeps")
   tinsert(instances, "Blackrock Depths")
   tinsert(instances, "Blackrock Spire")
   tinsert(instances, "Blackwing Lair")
   tinsert(instances, "Dire Maul")
   tinsert(instances, "Gnomeregan")
   tinsert(instances, "Maraudon")
   tinsert(instances, "Molten Core")
   tinsert(instances, "Naxxramas")
   tinsert(instances, "Onyxia's Lair")
   tinsert(instances, "Ragefire Chasm")
   tinsert(instances, "Razorfen Downs")
   tinsert(instances, "Razorfen Kraul")
   tinsert(instances, "Ruins of Ahn'Qiraj")
   tinsert(instances, "Scarlet Monastary")
   tinsert(instances, "Scholomance")
   tinsert(instances, "Shadowfang Keep")
   tinsert(instances, "Stratholme")       
   tinsert(instances, "The Deadmines")
   tinsert(instances, "The Stockade")
   tinsert(instances, "The Temple of Atal'Hakkar")
   tinsert(instances, "The Wailing Caverns")
   tinsert(instances, "Uldaman")   
   tinsert(instances, "Zul'Farrak")
   tinsert(instances, "Zul'Gurub")
   
   return instances
end

local function MTGetPVPs()
   PVPs = {}

   tinsert(PVPs, "Alterac Valley") 
   tinsert(PVPs, "Arathi Basin")   
   tinsert(PVPs, "Warsong Gulch")

   return PVPs

end

function MTContinentDropDownOnClick()
   UIDropDownMenu_SetSelectedID(MTContinentDropDown, this:GetID())

   -- current zone
   if gCurrSetup ~= 1 and MTContinentDropDown.selectedID == 2 then

      zones = MTGetInstances()
      idx = MTFindInArray(zones, gLastZone)
      if idx > 0 then
         gSetupList[gCurrSetup].autoload = getn(continents)-1
         gSetupList[gCurrSetup].zone = idx
      end

      if idx == 0 then
         zones = MTGetPVPs()
         idx = MTFindInArray(zones, gLastZone)
         if idx > 0 then
            gSetupList[gCurrSetup].autoload = getn(continents)
            gSetupList[gCurrSetup].zone = idx
         end
      end

      if idx == 0 then
         currcont = GetCurrentMapContinent()
         gSetupList[gCurrSetup].autoload = currcont+2
         zones = MTGetZones(GetMapZones(currcont))
         idx = MTFindInArray(zones, gLastZone)
         if idx > 0 then
            gSetupList[gCurrSetup].autoload = currcont+2
            gSetupList[gCurrSetup].zone = idx
         end
      end

      UIDropDownMenu_SetSelectedID(MTContinentDropDown, gSetupList[gCurrSetup].autoload, 1);
      --UIDropDownMenu_SetSelectedID(MTZoneDropDown, gSetupList[gCurrSetup].zone, 1)
      MTZoneDropDownInit()
   else
      gSetupList[gCurrSetup].autoload = MTContinentDropDown.selectedID
      gSetupList[gCurrSetup].zone = 0
      MTZoneDropDownInit()
   end

   if gSetupList[gCurrSetup].autoload == 1 or (gCurrSetup == 1 and gSetupList[1].autoload == 2) then
      MTZoneDropDown:Hide()
      return
   end
   
   MTZoneDropDown:Show()
end

local function MTGetContinents(...)
   continents = {}

   for i=1,arg.n do
      tinsert(continents, arg[i])
   end

   return continents
end

function MTContinentDropDownInit()
   local info

   UIDropDownMenu_ClearAll(MTContinentDropDown)
   DropDownList1.numButtons = 0

   if gCurrSetup == 1 then
      continents = {}
      tinsert(continents, "Load If Nothing Else Loads")
   else
      continents = MTGetContinents(GetMapContinents())
      tinsert(continents, "Instances")
      tinsert(continents, "PVP")
      tinsert(continents, 1, "Current Zone")
   end
   
   tinsert(continents, 1, "Don't Load")

   for i=1,getn(continents) do 
      info = {}
      info.text = continents[i]
      info.func = MTContinentDropDownOnClick
      UIDropDownMenu_AddButton(info)
   end

   UIDropDownMenu_SetSelectedID(MTContinentDropDown, gSetupList[gCurrSetup].autoload, 1);

   if gSetupList[gCurrSetup].autoload == 1 or (gCurrSetup == 1 and gSetupList[1].autoload == 2) then
      MTZoneDropDown:Hide()
   else
      MTZoneDropDown:Show()
   end
end

function MTLoadContinentDropDown()
   UIDropDownMenu_Initialize(MTContinentDropDown, MTContinentDropDownInit)
   UIDropDownMenu_SetWidth(95, MTContinentDropDown)
end

------------------- Zone drop down

function MTZoneDropDownOnClick()
   UIDropDownMenu_SetSelectedID(MTZoneDropDown, this:GetID())
   gSetupList[gCurrSetup].zone = MTZoneDropDown.selectedID
   gSetupList[gCurrSetup].zonename = MTZoneDropDownText:GetText()
end

function MTZoneDropDownInit()
   local info

   if MTContinentDropDown.selectedID == 1 then
      return
   end

   UIDropDownMenu_ClearAll(MTZoneDropDown)
   DropDownList1.numButtons = 0
   gZoneList = {}

   if MTContinentDropDown.selectedID == getn(continents)-1 then
      gZoneList = MTGetInstances()
   elseif MTContinentDropDown.selectedID == getn(continents) then  
      gZoneList = MTGetPVPs()
   else
      gZoneList = MTGetZones(GetMapZones(MTContinentDropDown.selectedID-2))
   end

   for i=1,getn(gZoneList) do 
      info = {}
      info.text = gZoneList[i]
      info.func = MTZoneDropDownOnClick
      UIDropDownMenu_AddButton(info)
   end

   if gSetupList[gCurrSetup].zone > 0 then
      UIDropDownMenu_SetSelectedID(MTZoneDropDown, gSetupList[gCurrSetup].zone, 1)
   end
end

gZoneList = nil

function MTLoadZoneDropDown()
   UIDropDownMenu_Initialize(MTZoneDropDown, MTZoneDropDownInit)
   UIDropDownMenu_SetWidth(85, MTZoneDropDown)
end

gLastZone = nil

local function MTListFindZone(zone)
   for i=1,getn(gSetupList) do
      if gSetupList[i].zonename == zone then
         return i
      end
   end

   return 0
end

function MTWorldUpdate()
   currzone = GetRealZoneText()

   lastidx = MTListFindZone(gLastZone)
   nowidx = MTListFindZone(currzone)

   --message(gLastZone..currzone)

   if nowidx > 0 and lastidx ~= nowidx then
      MTSaveCurrentSetup()
      gCurrSetup = nowidx
      MTSetCurrentSetup()
      MTSetupDropDownInit()
      MTContinentDropDownInit()
      MTZoneDropDownInit()
      MTLoad()
      MTShow()
   else
      if lastidx > 0 and lastidx ~= 1 and gSetupList[1].autoload == 2 then
         MTSaveCurrentSetup()
         gCurrSetup = 1
         MTSetCurrentSetup()
         MTSetupDropDownInit()
         MTContinentDropDownInit()
         MTZoneDropDownInit()
         MTLoad()
         MTShow()
      end
   end

   if not UnitAffectingCombat("player") and not MTPlayerDead() then

      -- Plaguelands check

      dontask = not gOpts.AskArgentDawn
      if not dontask then
         _,_,_,dontask = MTFindInLists("Seal of the Dawn")
      end
      if not dontask then
         _,_,_,dontask = MTFindInLists("Rune of the Dawn")
      end
      if not dontask then
         _,_,_,dontask = MTFindInLists("Argent Dawn Commission")
      end

      if MTJustEnteredUndead() and not dontask then
         itemLink = GetInventoryItemLink("player", 13)
         _,_,trinket1name = string.find(itemLink or "","item:%d+.+%[(.+)%]")
         itemLink = GetInventoryItemLink("player", 14)
         _,_,trinket2name = string.find(itemLink or "","item:%d+.+%[(.+)%]")

         if not string.find(trinket1name..trinket2name, "Argent Dawn Commission") and
            not string.find(trinket1name..trinket2name, "Seal of the Dawn") and
            not string.find(trinket1name..trinket2name, "Rune of the Dawn") then
            MTDialog("Would you like to equip your Argent Dawn trinket?", FlagEquipADTrinket)
         else
            ADAlreadyEquipped = false
         end
      end
   end

   gLastZone = currzone
end


gSpecialCases = {

["Talisman of Arathor"]          = "Spell_Holy_DevotionAura",
["Defiler's Talisman"]           = "Spell_Holy_DevotionAura",
["Talisman of Ephemeral Power"]  = "Spell_Holy_MindVision",
["Zandalarian Hero Charm"]       = "Spell_Lightning_LightningBolt01",
["Fetish of the Sand Reaver"]    = "Spell_Magic_LesserInvisibilty",
["Eye of Moan"]                  = "INV_Misc_AhnQirajTrinket_04",
["Mind Quickening Gem"]          = "Spell_Nature_WispHeal"

}

local function MTSpecialCase(slot)

   checkTrinket = nil

   for key, value in pairs(gSpecialCases) do
      _,idx,listtype = MTFindInLists(key)

      if idx >= 0 then
         if listtype == "Passive" then
            if gPassiveTrinkets[idx].slot == slot then
               checkTrinket = key
               break
            end         
         else
            if gCooldownTrinkets[idx].slot == slot then
               checkTrinket = key
               break
            end
         end
      end
   end

   if not checkTrinket then
      return false
   end
   
   for i=0,15 do
      -- check for special buffs for trinkets
      buffIdx = GetPlayerBuff(i, "HELPFUL")
      
      if buffIdx >= 0 then
         icon = GetPlayerBuffTexture(buffIdx)
         icon = string.sub(icon, 17)

         if icon == gSpecialCases[checkTrinket] then
            if GetPlayerBuffTimeLeft(buffIdx) > 0 then
               return true
            end
         end
      end
	end

   return false
end

-- cooldowns

totalcooldownelapsed = 0

function MTUpdateCooldown(elapsed)

   totalcooldownelapsed = totalcooldownelapsed + elapsed

   if totalcooldownelapsed < 1.0 then
   --   return
   end

   _,_,typ,bnum = string.find(this:GetName(),"MT(%a+)(%d+)")

   bnum = tonumber(bnum)

   cooldown = getglobal(this:GetName().."Cooldown")

   if typ == "Equipped" then
      start, duration, enable = GetInventoryItemCooldown("player", bnum+12)	   
   elseif typ == "Passive" or typ == "PassivePalette" then
      if bnum > gNumPassiveTrinkets then
         return
      end
      if gPassiveTrinkets[bnum-1].bag == WEARING then
         start, duration, enable = GetInventoryItemCooldown("player", 12-gPassiveTrinkets[bnum-1].slot)
      else
         start, duration, enable = GetContainerItemCooldown(gPassiveTrinkets[bnum-1].bag, gPassiveTrinkets[bnum-1].slot)
      end
   else -- "Cooldown"
      if bnum > gNumCooldownTrinkets then
         return
      end

      if gCooldownTrinkets[bnum-1].bag == WEARING then
         start, duration, enable = GetInventoryItemCooldown("player", 12-gCooldownTrinkets[bnum-1].slot)
      else
         start, duration, enable = GetContainerItemCooldown(gCooldownTrinkets[bnum-1].bag, gCooldownTrinkets[bnum-1].slot)
      end
      --message("setting "..start..", "..duration.." for bag "..gCooldownTrinkets[bnum-1].bag.." slot "..gCooldownTrinkets[bnum-1].slot)
   end

   CooldownFrame_SetTimer(cooldown, start, duration, enable)

   totalcooldownelapsed = 0
end

-- main update loop

totalupdateelapsed = 0.0
updateframe = 0.0
sincebegan = 0.0

function MTUpdate(updateelapsed)

   sincebegan = sincebegan + updateelapsed

   totalupdateelapsed = totalupdateelapsed + updateelapsed

   gUseTimer1 = gUseTimer1 + updateelapsed
   gUseTimer2 = gUseTimer2 + updateelapsed

   if (totalupdateelapsed < 1.0) then
      return
   end

   updateframe = updateframe + 1.0

   trinket1 = nil
   trinket2 = nil

   local foundIdx = nil
   
   -- find the 2 equipped trinkets (if any) in our lists
   itemLink = GetInventoryItemLink("player", 13)
    _,_,trinketid1,trinket1name = string.find(itemLink or "","item:(%d+).+%[(.+)%]")
   if trinketid1 then
      --start, duration = GetInventoryItemCooldown("player", 13)

      for i=0,gNumPassiveTrinkets-1 do         
         if gPassiveTrinkets[i].slot == TRINKETSLOT1 then
            foundIdx = i
            break
         end
      end

      if not foundIdx then
         for i=0,gNumCooldownTrinkets-1 do         
            if gCooldownTrinkets[i].slot == TRINKETSLOT1 then
               foundIdx = i
               break
            end
         end
         type1 = "Cooldown"
         trinket1 = foundIdx
      else
         type1 = "Passive"
         trinket1 = foundIdx
      end
   end

   foundIdx = nil

   itemLink = GetInventoryItemLink("player", 14)
    _,_,trinketid2,trinket2name = string.find(itemLink or "","item:(%d+).+%[(.+)%]")
   if trinketid2 then
      for i=0,gNumPassiveTrinkets-1 do         
         if gPassiveTrinkets[i].slot == TRINKETSLOT2 then
            foundIdx = i
            break
         end
      end

      if not foundIdx then         
         for i=0,gNumCooldownTrinkets-1 do         
            if gCooldownTrinkets[i].slot == TRINKETSLOT2 then
               foundIdx = i
               break
            end
         end
         type2 = "Cooldown"
         trinket2 = foundIdx
      else
         type2 = "Passive"
         trinket2 = foundIdx
      end
   end

   if not UnitAffectingCombat("player") and not MTPlayerDead() then
      -- check to see if the equipped trinkets have a cooldown

      if not gLock1 and not MTSpecialCase(TRINKETSLOT1) and gUseTimer1 > 2.0 then
         
         local success = false

         -- equip carrot if player is on mount
         if gOpts.AutoEquipCarrot and MTIsOnMount() then
            success = EquipCarrot(13)
         end

         if gEquipADTrinket1 then
            
            if ADEquippedTime and (GetTime()-ADEquippedTime) > 5.0 and 
               not string.find("Rune of the DawnSeal of the DawnArgent Dawn Commision", MTGetEquippedName(13)) then
               gEquipADTrinket1 = false
               success = false
            end

            if gEquipADTrinket1 and not success then
               success = EquipADTrinket(13)
            end
         end

         if not success then
            if trinketid1 then
               if type1 == "Passive" then
                  if gCooldownsOn then
                     success = EquipFirstCooldown(13)
                  end
                  -- get the best passive one then
                  if not success then
                     success = EquipFirstPassive(13, trinket1)
                  end
               else
                  -- if cooldowns are off, we need to switch out with passives
                  if not gCooldownsOn then
                     success = EquipFirstPassive(13)
                  else
                     -- only way we'd equip now is if there was something better ready
                     success = EquipFirstCooldown(13, trinket1)

                     if not success then
                        equipstart, equipduration = GetInventoryItemCooldown("player", 13)
                        -- if duration is greater than 30s it needs to be swapped
                        if GetTime() < equipstart+equipduration-30 then
                           if gCooldownsOn then 
                              success = EquipFirstCooldown(13)
                           end
                           if gPassivesOn and not success then
                              success = EquipFirstPassive(13)
                           end
                        end
                     end
                  end
               end
            else
               -- just equip first cooldown or passive
               if gCooldownsOn then 
                  success = EquipFirstCooldown(13)               
               end
               if gPassivesOn and not success then
                  success = EquipFirstPassive(13)
               end
            end
         end

         if success and not ADAlreadyEquipped then
            gAutoTrinketEquip1 = true
         end            
      end

      if not gLock2 and not MTSpecialCase(TRINKETSLOT2) and gUseTimer2 > 2.0 then
         
         local success = false

         -- equip carrot if player is on mount
         if gOpts.AutoEquipCarrot and MTIsOnMount() then
            success = EquipCarrot(14)
         end

         if gEquipADTrinket2 then
            
            if ADEquippedTime and (GetTime()-ADEquippedTime) > 5.0 and 
               not string.find("Rune of the DawnSeal of the DawnArgent Dawn Commision", MTGetEquippedName(14)) then
               gEquipADTrinket2 = false
               success = false
            end

            if gEquipADTrinket2 and not success then
               success = EquipADTrinket(14)
            end
         end
         
         if not success then
            if trinketid2 then
               if type2 == "Passive" then
                  if gCooldownsOn then
                     success = EquipFirstCooldown(14)
                  end
                  -- get the best passive one then
                  if not success then
                     success = EquipFirstPassive(14, trinket2)
                  end
               else
                  -- if cooldowns are off, we need to switch out with passives
                  if not gCooldownsOn then
                     success = EquipFirstPassive(14)
                  else
                     -- only way we'd equip now is if there was something better ready
                     success = EquipFirstCooldown(14, trinket2)

                     if not success then
                        equipstart, equipduration = GetInventoryItemCooldown("player", 14)
                        -- if duration is greater than 30s it needs to be swapped
                        if GetTime() < equipstart+equipduration-30 then
                           if gCooldownsOn then 
                              success = EquipFirstCooldown(14)
                           end
                           if gPassivesOn and not success then
                              success = EquipFirstPassive(14)
                           end
                        end
                     end
                  end
               end
            else
               -- just equip first cooldown or passive
               if gCooldownsOn then 
                  success = EquipFirstCooldown(14)               
               end
               if gPassivesOn and not success then
                  success = EquipFirstPassive(14)
               end  
            end
         end

         if success and not ADAlreadyEquipped then
            gAutoTrinketEquip2 = true
         end 
      end
   end

   MTLoad()
   MTShow()

   totalupdateelapsed = 0

end

-- event handler

function MTHandleEvent(event, arg1, arg2)
   if event == "PLAYER_LOGIN" then
      this:RegisterEvent("BAG_UPDATE")
      this:RegisterEvent("MINIMAP_ZONE_CHANGED")
      this:RegisterEvent("PLAYER_LOGOUT")
      this:RegisterEvent("SPELLCAST_START")
      this:RegisterEvent("SPELLCAST_STOP")

      gLastZone = GetRealZoneText()
      
      MTFixOlderVersions()
      MTSetCurrentSetup()
      MTLoadSetupDropDown()
      MTLoadContinentDropDown()
      MTLoadZoneDropDown()
      MTUpdateAdvanced()
      MTOptionsButtonInit()
      MTLoadOptions()
      
      MTLoad()
      MTShow()

   elseif event == "MINIMAP_ZONE_CHANGED" then
      
      MTWorldUpdate()

   elseif event == "BAG_UPDATE" then
      MTLoad()
      if MTFrameEquipped:IsVisible() then
         MTShow()
      end
   elseif event == "PLAYER_LOGOUT" then
      MTSaveCurrentSetup()
   elseif event == "SPELLCAST_START" then
      if string.find(arg1 or "", "saber") then
         gMountStarted = true
         gMountStartTime = GetTime()
      end
   elseif event == "SPELLCAST_STOP" then
      gMountFinishTime = GetTime()
      if gMountFinishTime < gMountStartTime+2.8 then
         message(gMountFinishTime-gMountStartTime)
         gMountStarted = false
      end
   end
end


-- slash handlers

function MTSlashHelp(arg1)

   DEFAULT_CHAT_FRAME:AddMessage("[MehHelp] 1) Open up Setup and organize your trinkets into either the", 225, 225, 0)
   DEFAULT_CHAT_FRAME:AddMessage("[MehHelp] Passives list or the Cooldowns list.", 225, 225, 0)
   DEFAULT_CHAT_FRAME:AddMessage("[MehHelp] 2) Prioritize your cooldown trinkets (left to right, top to", 225, 225, 0)
   DEFAULT_CHAT_FRAME:AddMessage("[MehHelp] bottom, top left spot has highest priority.)", 225, 225, 0)
   DEFAULT_CHAT_FRAME:AddMessage("[MehHelp] 3) Note that if you don't want something to be managed by", 225, 225, 0)
   DEFAULT_CHAT_FRAME:AddMessage("[MehHelp] the mod you can put it in the Passives list at slot 3 or", 225, 225, 0)
   DEFAULT_CHAT_FRAME:AddMessage("[MehHelp] above and it will never be equipped.", 225, 225, 0)
   DEFAULT_CHAT_FRAME:AddMessage("[MehHelp] 4) Use the Palette like this: left click on a trinket to", 225, 225, 0)
   DEFAULT_CHAT_FRAME:AddMessage("[MehHelp] equip it into the 1st slot, right click to equip into the 2nd slot.", 225, 225, 0)

end

function MTSlashShow(arg1)
   if MTFrameEquipped:IsVisible() then
      MTFrameEquipped:Hide()
      MTFrameTrinkets:Hide()
      MTFramePalette:Hide()
   else
      MTFrameEquipped:Show()
      if MTShowSetup:GetChecked() then
         MTFrameTrinkets:Show()
      end
      if MTShowPalette:GetChecked() then
         MTFramePalette:Show()
      end
   end
end